var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import crudTypes from './actionTypes';
/**
 * @todo add error action and make functions return that error when the requests don't succeed.
 */
/**
 * The RESTReduxCRUD module contains four redux objects: actions, actionTypes, reducers, and selectors.
 * RESTReduxCRUD has a peer dependency of {@link https://www.npmjs.com/package/redux-thunk | Redux Thunk} to avoid importing Redux's dispatch functionality into each file that depends on dispatch.
 * @namespace RESTReduxCRUD
 */
/**
 * Dynamically namespaced RESTful Redux CRUD actions for Queries, Datasets and DataSources entities.
 * @memberof RESTReduxCRUD
 * @namespace
 * @param {string} namespace Desired namespace for these actions eg. "queries".
 * @param {path} restResource Rest path that the actions will live under eg. "/query".
 * @returns {object} Set of CRUD actions accessible within the actions object.
 * @see See the related {@link RESTReduxCRUD.selectors|selectors} to check on action status and returned results where applicable.
 */
const actions = (namespace, restResource) => {
    // Setup the namespace on the actionTypes
    const actionTypes = crudTypes(namespace);
    // Synchronous actions. Each of these returns a namespaced action eg. {type: `queries/LIST_RESP`, payload: {<entities>}}.
    // List Request and Response
    const listReq = () => ({ type: actionTypes.LIST_REQ });
    const listResp = (entities) => ({ type: actionTypes.LIST_RESP, payload: { entities } });
    // Add Request and Response
    const addReq = (entity) => ({ type: actionTypes.ADD_REQ, payload: { entity } });
    const addResp = (entity) => ({ type: actionTypes.ADD_RESP, payload: { entity } });
    // Edit Request and Response
    const editReq = (entity) => ({ type: actionTypes.EDIT_REQ, payload: { entity } });
    const editResp = (entity) => ({ type: actionTypes.EDIT_RESP, payload: { entity } });
    // Delete Request and Response
    const deleteReq = (entity) => ({ type: actionTypes.DELETE_REQ, payload: { entity } });
    const deleteResp = (entity) => ({ type: actionTypes.DELETE_RESP, payload: { entity } });
    const list = () => {
        return (dispatch) => __awaiter(this, void 0, void 0, function* () {
            dispatch(listReq());
            const response = yield fetch(restResource, { credentials: 'include' });
            const entities = yield response.json();
            return dispatch(listResp(entities));
        });
    };
    const add = (entity) => {
        return (dispatch) => __awaiter(this, void 0, void 0, function* () {
            dispatch(addReq(entity));
            const fetchOpt = {
                method: 'POST',
                headers: [
                    ['Accept', 'application/json'],
                    ['Content-Type', 'application/json']
                ],
                body: JSON.stringify(entity),
                credentials: 'include'
            };
            const response = yield fetch(restResource, fetchOpt);
            const jsonEntity = yield response.json();
            return dispatch(addResp(jsonEntity));
        });
    };
    const edit = (entity) => {
        return (dispatch) => __awaiter(this, void 0, void 0, function* () {
            dispatch(editReq(entity));
            const resource = `${restResource}/${entity._id}`;
            const fetchOpt = {
                method: 'PUT',
                headers: [
                    ['Accept', 'application/json'],
                    ['Content-Type', 'application/json']
                ],
                body: JSON.stringify(entity),
                credentials: 'include'
            };
            const response = yield fetch(resource, fetchOpt);
            const jsonEntity = yield response.json();
            return dispatch(editResp(jsonEntity));
        });
    };
    const del = (entity) => {
        return (dispatch) => __awaiter(this, void 0, void 0, function* () {
            dispatch(deleteReq(entity));
            const resource = `${restResource}/${entity._id}`;
            const fetchOpt = {
                method: 'DELETE',
                headers: [
                    ['Accept', 'application/json'],
                    ['Content-Type', 'application/json']
                ],
                body: JSON.stringify(entity),
                credentials: 'include'
            };
            yield fetch(resource, fetchOpt);
            return dispatch(deleteResp(entity));
        });
    };
    // Each of list, add, edit and del are accessed through the action.---() object and use the above request and response action pairs.
    return {
        /**
         * List the entities of type namespace.
         * @function list
         * @memberof RESTReduxCRUD.actions
         * @returns {function} Thenable promise.
         * @see Use the {@link RESTReduxCRUD.selectors.getListPending|getListPending} and {@link RESTReduxCRUD.selectors.getAll|getAll} selectors to get action status and results.
         */
        list,
        /**
         * Add the entity of type namespace to it's list of entities.
         * @function add
         * @memberof RESTReduxCRUD.actions
         * @param {object} entity The entity being added.
         * @returns {function} Thenable promise.
         * @see Use the {@link RESTReduxCRUD.selectors.getAddPending|getAddPending} selector to get action status.
         */
        add,
        /**
         * @todo return boolean for success or failure.
         */
        /**
         * Edit (replace) the entity of type namespace associated with entity.entityId.
         * @function edit
         * @memberof RESTReduxCRUD.actions
         * @param {object} entity New version of entity. Uses the entityId inside to find one to edit.
         * @returns {function} Thenable promise.
         * @see Use the {@link RESTReduxCRUD.selectors.getEditPending|getEditPending} selector to get action status.
         */
        edit,
        /** @todo Could just pass the entity ID in here instead of the entity, unless we are doing a deep comparison. */
        /** @todo Return a boolean for success and failure of the delete. */
        /**
         * Delete the entity of type namespace assoicated with entity.entityId.
         * @function del
         * @memberof RESTReduxCRUD.actions
         * @param {object} entity Entity to delete. Uses entityId to find one to delete.
         * @returns {function} Thenable promise.
         * @see Use the {@link RESTReduxCRUD.selectors.getDeletePending|getDeletePending} selector to get action status.
         */
        del
    };
};
export default actions;
//# sourceMappingURL=actions.js.map