import crudTypes from './actionTypes';
// TODO should still update the state.listPending (and others) to false (or whatever) if entities is empty.
/**
 * Hardcoded initial state for the Redux store concerned with pending REST async operations, the set of entities in <namespace>, and a selected entity.
 * @memberof RESTReduxCRUD
 * @typedef {object} defaultState
 * @property {boolean} defaultState.listPending=false True when a list request is pending. False otherwise.
 * @property {boolean} defaultState.getPending=false True when a get request is pending. False otherwise.
 * @property {boolean} defaultState.addPending=false True when an add request is pending. False otherwise.
 * @property {array} defaultState.pendingEdits="[]" The list of entities being edited.
 * @property {boolean} defaultState.deletePending=false True when a delete request is pending. False otherwise.
 * @property {array} defaultState.entities="[]" The list of entities in state.
 */
export const defaultState = {
    listPending: false,
    addPending: false,
    pendingEdits: [],
    deletePending: false,
    entities: []
};
/**
 * Reduces namespaced actions to changes in state. In each reducer a new state object is constructed and returned with specific portions updated according to what action the reducer is written for.
 * @memberof RESTReduxCRUD
 * @namespace
 * @param {string} namespace The namespace of our actionTypes.
 */
const reducers = (namespace) => {
    const actionTypes = crudTypes(namespace);
    // Generally, if a reducer for a given action is not provided with the correct data we return the original state.
    // The behaviors object is standard Redux, and behaves like a switch statement once initialized.
    const behaviors = {
        /**
         * Updates state.listPending to true.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The state.
         * @returns {object} State with listPending updated to true.
         */
        [actionTypes.LIST_REQ](state) {
            return Object.assign({}, state, { listPending: true });
        },
        /**
         * Updates state.listPending to false, and updates state.entities with new entities.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {array} entities The array of <namespace> entities.
         * @returns {object} Updates state.listPending to false and state.entities to entities.
         */
        [actionTypes.LIST_RESP](state, { entities }) {
            if (!entities)
                throw new Error('actionTypes.LIST_RESP must be called with an entities parameter.');
            return Object.assign({}, state, { listPending: false, entities });
        },
        /**
         * Updates state.addPending to the entity being added.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pending add entity.
         * @returns {object} Updates state.addPending to the entity being added.
         */
        [actionTypes.ADD_REQ](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.ADD_REQ must be called with an entity parameter.');
            return Object.assign({}, state, { addPending: entity });
        },
        /**
         * Updates state.addPending to false and adds the entity to state.entities.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pendind add entity.
         * @returns {object} Updates state.addPending to false and state.entities has entity added to it without duplication.
         */
        [actionTypes.ADD_RESP](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.ADD_RESP must be called with an entity parameter.');
            return Object.assign({}, state, { addPending: false, entities: [
                    entity,
                    ...state.entities
                ] });
        },
        /**
         * Updates state.editPending to the entity being edited.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pending edit entity.
         * @returns {object} Updates state.editPending to the entity being edited.
         */
        [actionTypes.EDIT_REQ](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.EDIT_REQ must be called with an entity parameter.');
            return Object.assign({}, state, { pendingEdits: [
                    ...state.pendingEdits,
                    entity
                ], entities: state.entities.map((e) => e._id === entity._id ? entity : e) });
        },
        /**
         * @todo Handle case where the entity is not found in state.entities. Throw away? Add a new entity?
         */
        /**
         * Updates a specific entity in state.entities as found by entity._id.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pending edit entity.
         * @returns {object} If entity can be found in state.entities, updates that entity in state.entities.
         */
        [actionTypes.EDIT_RESP](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.EDIT_RESP must be called with an entity parameter.');
            // replace the edited entity
            return Object.assign({}, state, { pendingEdits: state.pendingEdits.filter((e) => e._id !== entity._id), entities: state.entities.map((e) => e._id === entity._id ? entity : e) });
        },
        /**
         * Updates state.deletePending to entity.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pending delete entity.
         * @returns {object} Updates state.deletePending to entity.
         */
        [actionTypes.DELETE_REQ](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.DELETE_REQ must be called with an entity parameter.');
            return Object.assign({}, state, { deletePending: true });
        },
        /**
         * Updates state.entities by deleting entity.
         * @memberof RESTReduxCRUD.reducers
         * @function
         * @param {object} state The application state.
         * @param {object} entity The pending delete entity.
         * @returns {object} Updates state.deletePending to undefined, and removes entity from state.entities by id if it can be found in the state.entities array.
         */
        [actionTypes.DELETE_RESP](state, { entity }) {
            if (!entity)
                throw new Error('actionTypes.DELETE_RESP must be called with an entity parameter.');
            // filter out the deleted entity
            return Object.assign({}, state, { deletePending: false, entities: state.entities.filter((e) => e._id !== entity._id) });
        }
    };
    // Our default state is initialized in these default parameters.
    return (state = defaultState, action) => {
        // Get the appropriate reducer based on our action.type
        const behavior = behaviors[action.type];
        // Call the reducer on the action. Return unmodified state if the behaviour isn't found in this reducer.
        return behavior ? behavior(state, action.payload || {}) : state;
    };
};
export default reducers;
//# sourceMappingURL=reducers.js.map