/**
 * The selector functions return access to portions of the Redux store related to request and response statuses, as well as the resulting entities themselves.
 * Selectors are all namespaced to portions of the state that you define for individual entity types.
 * @memberof RESTReduxCRUD
 * @namespace
 * @param {string} namespace The namespace of our selectors. eg. Query
 * @returns {object} selectors object containing our selectors.
 */
const selectors = (namespace) => {
    const localState = (state) => state[namespace];
    return {
        /**
         * getListPending accesses the listPending state.
         * @memberof RESTReduxCRUD.selectors
         * @function
         * @param {object} state Application state.
         * @returns {boolean} listPending boolean.
         * @see Provides access to state changed by the {@link RESTReduxCRUD.actions.list|list} action.
         */
        getListPending: (state) => localState(state).listPending,
        /**
         * getAddPending accesses the addPending state.
         * @memberof RESTReduxCRUD.selectors
         * @function
         * @param {object} state Application state.
         * @returns {boolean} addPending boolean.
         * @see Provides access to state changed by the {@link RESTReduxCRUD.actions.add|add} action.
         */
        getAddPending: (state) => localState(state).addPending,
        /**
         * Returns the list of entities being edited on the back end.
         * @memberof RESTReduxCRUD.selectors
         * @function
         * @param {object} state Application state.
         * @returns {Array}
         * @see Provides access to state changed by the {@link RESTReduxCRUD.actions.edit|edit} action.
         */
        getPendingEdits: (state) => localState(state).pendingEdits,
        /**
         * getDeletePending accesses the deletePending state.
         * @memberof RESTReduxCRUD.selectors
         * @function
         * @param {object} state Application state.
         * @returns {boolean} deletePending boolean.
         * @see Provides access to state changed by the {@link RESTReduxCRUD.actions.del|del} action.
         */
        getDeletePending: (state) => localState(state).deletePending,
        /**
         * getAll accesses the namespace entities in state.
         * @memberof RESTReduxCRUD.selectors
         * @function
         * @param {object} state Application state.
         * @returns {array} entities array.
         * @see Provides access to the entities array requested by the {@link RESTReduxCRUD.actions.list|list} action. Also see the {@link RESTReduxCRUD.selectors.getListPending|getListPending} selector.
         */
        getAll: (state) => localState(state).entities
    };
};
export default selectors;
//# sourceMappingURL=selectors.js.map